#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Transparent Health

import os
import pysftp
from jdt.csv2mongo import csv2mongo
from jdt.ndjson2mongo import ndjson2mongo


def download_files_from_ftp(
        sftp_host,
        sftp_username,
        sftp_private_key_path,
        known_hosts_path,
        remote_path_to_download,
        local_destination_path=""):
    cnopts = pysftp.CnOpts()
    cnopts.hostkeys.load(known_hosts_path)
    # print("Connect to", myHostname, myUsername, private_key,)
    localpaths = []
    with pysftp.Connection(sftp_host, username=sftp_username,
                           private_key=sftp_private_key_path, cnopts=cnopts) as sftp:

        print("SFTP Connection successfully established...")
        response_dict = {}
        sftp.cwd(remote_path_to_download)

        # Switch to a remote directory

        # Obtain structure of the remote directory '/var/www/vhosts'
        directory_structure = sftp.listdir_attr()

        # Print data
        for attr in directory_structure:
            #print(attr.filename, attr)
            # Define the file that you want to download from the remote
            # directory
            remoteFilePath = attr.filename
            localFilePath = attr.filename
            if local_destination_path:
                localFilePath = os.path.join(
                    local_destination_path, localFilePath)
            sftp.get(remoteFilePath, localFilePath)
            localpaths.append(localFilePath)

    return localpaths


def import_files_into_mongo(list_of_downloaded_filepaths, database_name,
                            delete_collection_before_import, host, port):
    for path in list_of_downloaded_filepaths:
        splitpath = os.path.split(path)
        colection_name, extension = splitpath[-1].rsplit(".", 1)
        if path.endswith("ndjson"):
            colection_name = "%s-fhir" % (colection_name)
            print(ndjson2mongo(path, database_name, colection_name,
                               delete_collection_before_import, host, port))

        if path.endswith("csv"):
            colection_name = "%s-csv" % (colection_name)
            print(csv2mongo(path, database_name, colection_name,
                            delete_collection_before_import, host, port))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Alan Viars

import argparse
import json
import sys
from collections import OrderedDict
from pymongo import MongoClient
import ndjson


def ndjsonFHIRBundle2mongo(ndjsonfile, database_name,
                 delete_database_before_import,
                 host,
                 port,
                 collection_name_prefix=''):
    
    """Inserts Bundle as separate resources."""
    print("Opening NDJSON  file:", ndjsonfile,)
    print("NOTE: Depending on your computer/configuration, large files may need to be split.")
    print("""If you get a "Process killed" in AWS, try A.) splitting the files using "split" or B.) increase the machine's RAM/CPU.""")

    response_dict = OrderedDict()
    fileindex = 0
    mongoindex = 0
    error_list = []

    mc = MongoClient(host=host, port=port)
    db = mc[database_name]

    if delete_database_before_import:
        print("Dropping the database...", database_name)
        mc.drop_database(database_name)
    print("Opening NDJSON file. Expecting FHIR Bundles...")
    with open(ndjsonfile) as f:
        print("Loading the NDJSON into memory...")
        data = ndjson.load(f, object_pairs_hook=OrderedDict)
        print("Data loaded...")
        print("Processing items...")
        i=0
        for item in data:
            # print("item",item)
            i +=1
            try:
                if not isinstance(item, type(OrderedDict())):
                    error_message = "Line %s did not contain a JSON object." % (i)
                    error_list.append(error_message)
                # insert the item/document
                for e in item['entry']:
                    #print(mongoindex,e['resource']["resourceType"])
                    collection_name = "%s%s" % (collection_name_prefix,e['resource']["resourceType"])
                    collection = db[collection_name]    
                    myobjectid = collection.insert(e)
                    mongoindex += 1

            except:
                print(sys.exc_info())
                error_message = "Line %s - %s" % (i, sys.exc_info())
                error_list.append(error_message)

        if error_list:
            response_dict['file'] = ndjsonfile
            response_dict['database'] = database_name
            response_dict['num_files_imported'] = mongoindex
            response_dict['num_file_errors'] = len(error_list)
            response_dict['errors'] = error_list
            response_dict['code'] = 400
            response_dict['message'] = "Completed with errors."

        else:
            response_dict['file'] = ndjsonfile
            response_dict['database'] = database_name
            response_dict['num_rows_imported'] = mongoindex
            response_dict['num_file_errors'] = len(error_list)
            response_dict['code'] = 200
            response_dict['message'] = "Completed without errors."

    return response_dict

if __name__ == "__main__":

    # Parse args
    parser = argparse.ArgumentParser(
        description='Input NDJSON file.')
    parser.add_argument(
        dest='input_ndjson_file',
        action='store',
        help='Input the NDJSON file to load here')
    parser.add_argument(
        dest='db_name',
        action='store',
        help="Enter the Database name you want to import the JSON to")
    parser.add_argument('-d', '--delete', dest='delete', action='store_true',
                        help='Delete previous collection upon import')
    parser.add_argument(
        '--host',
        dest='host',
        action='store',
        default='127.0.0.1',
        help='Specify host. Default is 127.0.0.1 ')
    parser.add_argument(
        '-c', '--collection_name_prefix',
        dest='cnp',
        action='store',
        default='',
        help='Specify a prefix for FHIR resource collection names.Default is blank')
    parser.add_argument(
        '-p',
        '--port',
        dest='port',
        action='store',
        default=27017,
        help='Specify port. Default is 27017')
    args = parser.parse_args()
    ndjson_file = args.input_ndjson_file
    database = args.db_name
    delete_database_before_import = args.delete
    collection_name_prefix = args.cnp
    host = args.host
    port = args.port

    result = ndjsonFHIRBundle2mongo(
        ndjson_file,
        database,
        delete_database_before_import,
        host,
        port,
        collection_name_prefix)

    # output the JSON transaction summary
    print(json.dumps(result, indent=4))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Alan Viars

import argparse
import json
from collections import OrderedDict


def bundle_stub():
    od = OrderedDict()
    od["resourceType"] = "Bundle"
    od["type"] = "transaction"
    od["entry"] = []
    return od


def fhirbundledownsize(bundle_object, output_size):
    """FHIR Bundle to downsized Bundles with max entries of output_size"""

    response_list = []
    i = 0
    total = 0
    if "entry" not in bundle_object.keys():
        error = "Failed to find entry [] list. This file does not appear to be a FHIR Bundle."
        raise Exception(error)
    num_of_entries = len(bundle_object['entry'])
    for e in bundle_object['entry']:
        if i == 0:
            # Create a new Bundle
            bundle = bundle_stub()
            bundle['entry'].append(e)
        elif int(output_size) % i == 0:
            bundle = bundle_stub()
            bundle['entry'].append(e)
        else:
            # append a new entry
            bundle['entry'].append(e)

        if i == int(output_size):
            response_list.append(bundle)
            i = 0
        elif total == num_of_entries-1:
            response_list.append(bundle)
        i += 1
        total += 1

    return response_list


if __name__ == "__main__":

    parser = argparse.ArgumentParser(
        description='Load in FHIR Bunlde and save multiple FHIR Bundles.')
    parser.add_argument(
        dest='bundle_file',
        action='store',
        help='Input the FHIR Bundle.')
    parser.add_argument(
        dest='output_size',
        action='store',
        default="500",
        help="Enter the integer of the maximum number of entries per bundle.")
    args = parser.parse_args()

    in_fh = open(args.bundle_file, 'r')
    bundle_object = json.loads(in_fh.read(), object_pairs_hook=OrderedDict)
    print("The number of entries in the source file is %s" % (len(bundle_object['entry'])))
    result_list = fhirbundledownsize(bundle_object, args.output_size)
    i = 1
    for r in result_list:
        output_file = "%s_%s" % (args.bundle_file, i)
        out_fh = open(output_file, 'w')
        json.dump(r, out_fh)
        out_fh.close()
        print("%s file created." % (output_file))
        i += 1

    # output the JSON transaction summary
    print("%s FHIR Bundles created." % len(result_list))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Alan Viars

import argparse
import json
import os
import ndjson
import sys
from collections import OrderedDict


def jsonlist2ndjson(jsonlistinput_filename, output_filename):
    """Convert a single json list of json objects to a single ndjson file."""


    error_list = []
    success_index = 0
    response_dict = OrderedDict()

    try:
        # get the files in the specified directory
        print("Getting a list from from file", jsonlistinput_filename)
        in_fh = open(jsonlistinput_filename, 'r')

        # Writing the
        out_fh = open(output_filename, 'w')
        writer = ndjson.writer(out_fh)
        print("Outputting file", jsonlistinput_filename)
        reader = in_fh.read()
        thelist = json.loads(reader, object_pairs_hook=OrderedDict)

        for i in thelist:

            if not isinstance(i, type(OrderedDict())):
                error_message = "File " + f + " did not contain a json object, i.e. {}."
                print(error_message)
            else:
                writer.writerow(i)
                success_index+=1
        in_fh.close()
        out_fh.close()

    except Exception as e:
            print("error", e)
            error_message = e
            error_list.append(error_message)
            if not error_message:
                # add the object to the output file.
                try:
                    writer.writerow(j)
                    success_index += 1
                except:
                    error_message = "Error writing " + f + \
                        " to NDJSON. " + str(sys.exc_info())
                    error_list.append(error_message)
    
    response_dict = {"error_list": error_list, 
                     "input_file":  jsonlistinput_filename, 
                      "output_file":  output_filename,
                      "number_processed": success_index,
                      "number_in_input": len(thelist) 
                     }
    return response_dict


if __name__ == "__main__":
    
    # Parse args
    parser = argparse.ArgumentParser(
        description='Load in file with a JSON list that contains a JSON objects and convert it into a single NDJSON file.')

    parser.add_argument(
        dest='input_JSON',
        action='store',
        help='Input the JSON list to load here')

    parser.add_argument(
        dest='output_NDJSON_file',
        action='store',
        help="Enter the output NDJSON filename")

    args = parser.parse_args()

    result = jsonlist2ndjson(args.input_JSON, args.output_NDJSON_file)

    # output the JSON transaction summary
    print(json.dumps(result, indent=4))



import json

def converter(cs, SLAP_URL, scopesObject):
    title = ""
    description = ""
    if "id" in cs.keys():
        title = cs["id"]
    if "description" in cs.keys():
        description = cs["description"]

    swag = {
            "openapi": "3.0.2",
            "info": {
                "title": f'{title} Capability Statement',
                "description": description,
                "license": {
                    "name": "Creative Commons Zero v1.0 Universal",
                    "url": "http://spdx.org/licenses/CC0-1.0.html"
                },
                "version": "1.0.0",
                "contact": {
                    "name": "HL7 Financial Management Working Group",
                    "email": "fm@lists.HL7.org",
                    "url": "http://www.hl7.org/Special/committees/fm/index.cfm"
                }
            },
            "externalDocs": {
                "url": cs["url"],
                "description": "FHIR Capability Statement"
            },
            "paths": {
                "/metadata": {
                    "summary": "Access to the server's Capability Statement",
                    "get": {
                        "summary": "Return the Capability Statement",
                        "responses": {
                            "default": {
                                "description": "Error, with details",
                                "content": {
                                    "application/fhir+json": {
                                        "schema": {
                                            "$ref": "https://hl7.org/fhir/R4/fhir.schema.json#/definitions/OperationOutcome"
                                        }
                                    }
                                }
                            },
                            "200": {
                                "description": "The Capability Statement",
                                "content": {
                                    "application/fhir+json": {
                                        "schema": {
                                            "$ref": "https://hl7.org/fhir/R4/fhir.schema.json#/definitions/CapabilityStatement"
                                        }
                                    }
                                }
                            }
                        },
                        "parameters": [
                            {
                                "$ref": "#/components/parameters/format"
                            },
                            {
                                "$ref": "#/components/parameters/pretty"
                            },
                            {
                                "$ref": "#/components/parameters/summary"
                            },
                            {
                                "$ref": "#/components/parameters/elements"
                            }
                        ]
                    }
                }
            }
        }

    components = {
      "parameters": {
        "rid": {
          "name": "rid",
          "in": "path",
          "description": "id of the resource (\u003dResource.id)",
          "required": True,
          "allowEmptyValue": False,
          "style": "simple",
          "schema": {
            "type": "string"
          }
        },
        "hid": {
          "name": "hid",
          "in": "path",
          "description": "id of the history entry (\u003dResource.meta.versionId)",
          "required": True,
          "allowEmptyValue": False,
          "style": "simple",
          "schema": {
            "type": "string"
          }
        },
        "summary": {
          "name": "_summary",
          "in": "query",
          "description": "Requests the server to return a designated subset of the resource",
          "allowEmptyValue": True,
          "style": "form",
          "schema": {
            "type": "string",
            "enum": [
              "true",
              "text",
              "data",
              "count",
              "false"
            ]
          }
        },
        "format": {
          "name": "_format",
          "in": "query",
          "description": "Specify alternative response formats by their MIME-types (when a client is unable acccess accept: header)",
          "allowEmptyValue": True,
          "style": "form",
          "schema": {
            "type": "string",
            "format": "mime-type"
          }
        },
        "pretty": {
          "name": "_pretty",
          "in": "query",
          "description": "Ask for a pretty printed response for human convenience",
          "allowEmptyValue": True,
          "style": "form",
          "schema": {
            "type": "boolean"
          }
        },
        "elements": {
          "name": "_elements",
          "in": "query",
          "description": "Requests the server to return a collection of elements from the resource",
          "allowEmptyValue": True,
          "style": "form",
          "explode": False,
          "schema": {
            "type": "array",
            "format": "string",
            "items": {
              "format": "string"
            }
          }
        },
        "count": {
          "name": "_count",
          "in": "query",
          "description": "The maximum number of search results on a page. The server is not bound to return the number requested, but cannot return more",
          "schema": {
            "type": "number"
          }
        }
      }
    }

    security = [{
        "type": "oauth2",
        "description": "Smart on FHIR implementation of Oauth2",
        "name": "Authorization",
        "in": "header",
        "scheme": "bearer",
        "bearerFormat": "JWT",
        "flows": [{
            "authorizationCode": {
                "authorizationUrl": f'{SLAP_URL}/authorize',
                "tokenUrl": f'{SLAP_URL}/token',
                "scopes": scopesObject
            }
        }]
    }]

    for rest in cs['rest']:
        # find the rest object that represents the server
        if rest["mode"] == "server":
            for resource in rest["resource"]:
                # iterate over every resource and create a paths object for every interaction it has.  use existing openapi sample for bootstrap
                for i in resource["interaction"]:
                    if i["code"] == "read":
                        swag['paths'][f'/{resource["type"]}/' + '{rid}'] = {
                                "get": {
                                    "description": f'Returns a specified {resource["type"]} resource with a given resource id',
                                    "responses": {
                                        "default": {
                                            "description":"Error, with details",
                                            "content":{
                                                "application/fhir+json":{
                                                    "schema":{
                                                        "$ref":"https://hl7.org/fhir/R4/fhir.schema.json#/definitions/OperationOutcome"
                                                    }
                                                }
                                            }
                                        },
                                        "200": {
                                            "description": f'A {resource["type"]} with specified resource id',
                                            "content":{
                                                "application/fhir+json":{
                                                    "schema":{
                                                        "$ref":"https://hl7.org/fhir/R4/fhir.schema.json#/definitions/OperationOutcome"
                                                    }
                                                }
                                            }
                                        }
                                    },
                                    "security": {
                                        "oauth2": [
                                            f'patient/{resource["type"]}.read'
                                        ]
                                    },
                                    "parameters":[
                                        {
                                            "$ref":"#/components/parameters/rid"
                                        },
                                        {
                                            "$ref":"#/components/parameters/summary"
                                        },
                                        {
                                            "$ref":"#/components/parameters/format"
                                        },
                                        {
                                            "$ref":"#/components/parameters/pretty"
                                        },
                                        {
                                            "$ref":"#/components/parameters/elements"
                                        }
                                    ]
                                }
                            }
                    elif i["code"] == "vread":
                         swag['paths'][f'/{resource["type"]}/' + '{rid}/_history/{hid}'] = {
                             "get": {
                                 "description": f'Returns a specified {resource["type"]} resource with a given resource id and a specific historical version',
                                 "responses": {
                                     "default": {
                                         "description":"Error, with details",
                                         "content":{
                                             "application/fhir+json":{
                                                 "schema":{
                                                     "$ref":"https://hl7.org/fhir/R4/fhir.schema.json#/definitions/OperationOutcome"
                                                 }
                                             }
                                         }
                                     },
                                     "200": {
                                         "description": f'A {resource["type"]} with specified resource id and specified version id',
                                         "content":{
                                             "application/fhir+json":{
                                                 "schema":{
                                                     "$ref":"https://hl7.org/fhir/R4/fhir.schema.json#/definitions/OperationOutcome"
                                                 }
                                             }
                                         }
                                     }
                                 },
                                "security": {
                                    "oauth2": [
                                        f'patient/{resource["type"]}.read'
                                    ]
                                },
                                 "parameters":[
                                     {
                                         "$ref":"#/components/parameters/rid"
                                     },
                                     {
                                         "$ref":"#/components/parameters/hid"
                                     },
                                     {
                                         "$ref":"#/components/parameters/summary"
                                     },
                                     {
                                         "$ref":"#/components/parameters/format"
                                     },
                                     {
                                         "$ref":"#/components/parameters/pretty"
                                     },
                                     {
                                         "$ref":"#/components/parameters/elements"
                                     }
                                 ]
                             }
                         }
                    elif i["code"] == "history-instance":
                         swag['paths'][f'/{resource["type"]}/' + '{rid}/_history'] = {
                             "get": {
                                 "description": f'Returns a bundle of all versions for the {resource["type"]} resource with given resource id',
                                 "responses": {
                                     "default": {
                                         "description":"Error, with details",
                                         "content":{
                                             "application/fhir+json":{
                                                 "schema":{
                                                     "$ref":"https://hl7.org/fhir/R4/fhir.schema.json#/definitions/OperationOutcome"
                                                 }
                                             }
                                         }
                                     },
                                     "200": {
                                         "description": f'A bundle of all versions for the {resource["type"]} resource with specified resource id',
                                         "content":{
                                             "application/fhir+json":{
                                                 "schema":{
                                                     "$ref":"https://hl7.org/fhir/R4/fhir.schema.json#/definitions/OperationOutcome"
                                                 }
                                             }
                                         }
                                     }
                                 },
                                "security": {
                                    "oauth2": [
                                        f'patient/{resource["type"]}.read'
                                    ]
                                },
                                 "parameters":[
                                     {
                                         "$ref":"#/components/parameters/rid"
                                     },
                                     {
                                         "$ref":"#/components/parameters/summary"
                                     },
                                     {
                                         "$ref":"#/components/parameters/format"
                                     },
                                     {
                                         "$ref":"#/components/parameters/pretty"
                                     },
                                     {
                                         "$ref":"#/components/parameters/elements"
                                     }
                                 ]
                             }
                         }
                    elif i["code"] == "history-type":
                         swag['paths'][f'/{resource["type"]}/_history'] = {
                             "get": {
                                 "description": f'Returns a bundle of all versions for the {resource["type"]} resources that match provided search parameters',
                                 "responses": {
                                     "default": {
                                         "description":"Error, with details",
                                         "content":{
                                             "application/fhir+json":{
                                                 "schema":{
                                                     "$ref":"https://hl7.org/fhir/R4/fhir.schema.json#/definitions/OperationOutcome"
                                                 }
                                             }
                                         }
                                     },
                                     "200": {
                                         "description": f'A bundle of all versions for the {resource["type"]} resource s that match provided search parameters',
                                         "content":{
                                             "application/fhir+json":{
                                                 "schema":{
                                                     "$ref":"https://hl7.org/fhir/R4/fhir.schema.json#/definitions/OperationOutcome"
                                                 }
                                             }
                                         }
                                     }
                                 },
                                "security": {
                                    "oauth2": [
                                        f'user/{resource["type"]}.read'
                                    ]
                                },
                                 "parameters":[
                                     {
                                         "$ref":"#/components/parameters/summary"
                                     },
                                     {
                                         "$ref":"#/components/parameters/format"
                                     },
                                     {
                                         "$ref":"#/components/parameters/pretty"
                                     },
                                     {
                                         "$ref":"#/components/parameters/elements"
                                     }
                                 ]
                             }
                         }
                         if "searchParam" in i.keys():
                             for param in i["searchParam"]:
                                 swag['paths'][f'/{resource["type"]}/_history']["get"]["parameters"].append({
                                     "$ref":f'#components/parameters/{param["name"]}'
                                 })
                                 components["parameters"][param["name"]] = {
                                      "name": param["name"],
                                      "in": "path",
                                      "required": False,
                                      "allowEmptyValue": False,
                                      "description": param["definition"],
                                      "schema": {
                                        "type": param["type"]
                                      }
                                    }
                    elif i["code"] == "search-type":
                         swag['paths'][f'/{resource["type"]}'] = {
                             "get": {
                                 "description": f'Returns a bundle of all versions for the {resource["type"]} resources that match provided search parameters',
                                 "responses": {
                                     "default": {
                                         "description":"Error, with details",
                                         "content":{
                                             "application/fhir+json":{
                                                 "schema":{
                                                     "$ref":"https://hl7.org/fhir/R4/fhir.schema.json#/definitions/OperationOutcome"
                                                 }
                                             }
                                         }
                                     },
                                     "200": {
                                         "description": f'A bundle of all versions for the {resource["type"]} resource s that match provided search parameters',
                                         "content":{
                                             "application/fhir+json":{
                                                 "schema":{
                                                     "$ref":"https://hl7.org/fhir/R4/fhir.schema.json#/definitions/OperationOutcome"
                                                 }
                                             }
                                         }
                                     }
                                 },
                                "security": {
                                    "oauth2": [
                                        f'patient/{resource["type"]}.read'
                                    ]
                                },
                                 "parameters":[
                                     {
                                         "$ref":"#/components/parameters/summary"
                                     },
                                     {
                                         "$ref":"#/components/parameters/format"
                                     },
                                     {
                                         "$ref":"#/components/parameters/pretty"
                                     },
                                     {
                                         "$ref":"#/components/parameters/elements"
                                     }
                                 ]
                             }
                         }
                         if "searchParam" in i.keys():
                             for param in i["searchParam"]:
                                 swag['paths'][f'/{resource["type"]}']["get"]["parameters"].append({
                                     "$ref":f'#components/parameters/{param["name"]}'
                                 })
                                 components["parameters"][param["name"]] = {
                                      "name": param["name"],
                                      "in": "path",
                                      "required": False,
                                      "allowEmptyValue": False,
                                      "description": param["definition"],
                                      "schema": {
                                        "type": param["type"]
                                      }
                                    }
                    # continue to add each of the following interactions
                    # elif i["code"] == "update":
                    # elif i["code"] == "patch":
                    # elif i["code"] == "delete":
                    # elif i["code"] == "create":
                    else:
                        # skip system interactions (not related to any resource)
                        pass
        else:
            # ignoring the client object, if its even there (unlikely)
            pass
    
    swag["security"] = security
    swag["components"] = components
    return swag
    

if __name__ == "__main__":
    cs = json.loads(open("sampleCS.json", 'r').read())
    scopes = {
      "fhirUser":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/fhirUser"
      ],
      "openid":[
         "openid"
      ],
      "system/*.*":[
         
      ],
      "system/*.read":[
         
      ],
      "system/*.write":[
         
      ],
      "patient/*.*":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FOrganization.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FOrganization.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FPatient.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FOrganization.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FPatient.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FPractitioner.read"
      ],
      "patient/*.read":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FOrganization.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FOrganization.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FPatient.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FOrganization.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FPatient.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FPractitioner.read"
      ],
      "patient/*.write":[
         
      ],
      "user/*.*":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FOrganization.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FOrganization.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FPatient.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FOrganization.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FPatient.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FPractitioner.read"
      ],
      "user/*.read":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FOrganization.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FOrganization.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FPatient.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FCoverage.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FExplanationOfBenefit.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FOrganization.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FPatient.read",
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FPractitioner.read"
      ],
      "user/*.write":[
         
      ],
      "patient/Coverage.read":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FCoverage.read"
      ],
      "patient/Coverage.write":[
         
      ],
      "user/Coverage.read":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FCoverage.read"
      ],
      "user/Coverage.write":[
         
      ],
      "patient/Coverage.*":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FCoverage.read"
      ],
      "user/Coverage.*":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FCoverage.read"
      ],
      "patient/ExplanationOfBenefit.read":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FExplanationOfBenefit.read"
      ],
      "patient/ExplanationOfBenefit.write":[
         
      ],
      "user/ExplanationOfBenefit.read":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FExplanationOfBenefit.read"
      ],
      "user/ExplanationOfBenefit.write":[
         
      ],
      "patient/ExplanationOfBenefit.*":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FExplanationOfBenefit.read"
      ],
      "user/ExplanationOfBenefit.*":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FExplanationOfBenefit.read"
      ],
      "patient/Organization.read":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FOrganization.read"
      ],
      "patient/Organization.write":[
         
      ],
      "user/Organization.read":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FOrganization.read"
      ],
      "user/Organization.write":[
         
      ],
      "patient/Organization.*":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FOrganization.read"
      ],
      "user/Organization.*":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FOrganization.read"
      ],
      "patient/Patient.read":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FPatient.read"
      ],
      "patient/Patient.write":[
         
      ],
      "user/Patient.read":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FPatient.read"
      ],
      "user/Patient.write":[
         
      ],
      "patient/Patient.*":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FPatient.read"
      ],
      "user/Patient.*":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FPatient.read"
      ],
      "patient/Practitioner.read":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FPractitioner.read"
      ],
      "patient/Practitioner.write":[
         
      ],
      "user/Practitioner.read":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FPractitioner.read"
      ],
      "user/Practitioner.write":[
         
      ],
      "patient/Practitioner.*":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/patient%2FPractitioner.read"
      ],
      "user/Practitioner.*":[
         "https://nwsfdevrbyhtenantb2c.onmicrosoft.com/nw-sf-dev-uses0-rbyh-safhirapi-app/user%2FPractitioner.read"
      ]
    }
    scopesObject = {
        "fhirUser": "required scope",
        "openid": "required scope",
        "patient/*.*": "perform all actions for a given patient",
        "patient/*.read": "read all resources for a given patient",
        "user/*.*": "perform all actions for multiple patients",
        "user/*.read": "read all resources for multiple patients",
    }
    for key in scopes.keys():
        if len(scopes[key]) != 0 and key not in ["fhirUser", "openid", "patient/*.*", "user/*.*", "patient/*.read", "user/*.read", "user/*.write", "patient/*.write"]:
            scopesObject[key] = f'{key.split(".")[-1]} the {(key.split("/")[-1]).split(".")[0]} resource'
    
    data = converter(cs, "https://slap.com", scopesObject)
    print(json.dumps(data, indent=4))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Alan Viars

import argparse
from itertools import zip_longest


def grouper(n, iterable, fillvalue=None):
    "Collect data into fixed-length chunks or blocks"
    # grouper(3, 'ABCDEFG', 'x') --> ABC DEF Gxx
    args = [iter(iterable)] * n
    return zip_longest(fillvalue=fillvalue, *args)


def split_large_files(filename, number_of_lines_per_file):
    with open(filename) as f:
        for i, g in enumerate(grouper(number_of_lines_per_file, f, fillvalue=''), 1):
            small_filename = '%s_%s' % (filename, format(i * number_of_lines_per_file))
            with open(small_filename, 'w') as fout:
                fout.writelines(g)


if __name__ == "__main__":

    # Parse args
    parser = argparse.ArgumentParser(
        description='Split a large file into many based on a specified number of lines.')
    parser.add_argument(
        dest='input_file',
        action='store',
        help='Input file to be split')
    parser.add_argument(
        dest='number_of_lines_per_file',
        action='store',
        help="Enter the numbert of lines per file.")

    args = parser.parse_args()
    split_large_files(args.input_file, int(args.number_of_lines_per_file))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Alan Viars

import argparse
import json
import sys
from collections import OrderedDict
import ndjson


def fhirbundle2ndjson(bundle_file, output_file):
    
    """FHIR Bundle to separate resources."""
    # print("Opening FHIR Bundle file:", bundle_file,)
  
    response_dict = OrderedDict()
    i = 0
    error_list = []
    
    out_fh = open(output_file, 'w')
    writer = ndjson.writer(out_fh)
    in_fh = open(bundle_file, 'r')
        
    response_dict['input_file'] = bundle_file
    response_dict['num_entries_converted'] = i
    response_dict['num_errors'] = 0

    item = json.loads(in_fh.read(), object_pairs_hook=OrderedDict)
    # print("JSON loaded successfully...")

    if "entry" not in item.keys():
            error = "Failed to find entry [] list."
            # print(error)
            error_list.append(error)
            response_dict['num_errors'] = len(error_list)
            response_dict['errors'] = error_list
            response_dict['code'] = 400            
            response_dict['message'] = "Completed with errors."
            return response_dict
    i=0
    for e in item['entry']:
        try:
            writer.writerow(e['resource'])
            i+=1
        except Exception as e:
            error_message = "Error writing " + e + \
                    " to NDJSON. " + str(sys.exc_info(), + str(e))
            error_list.append(error_message)
            
    response_dict['input_file'] = bundle_file
    response_dict['output_file'] = output_file
    response_dict['errors'] = error_list
    response_dict['num_entries_imported'] = i
    response_dict['num_errors'] = len(error_list)
        
    if response_dict['num_errors']:
        response_dict['code'] = 400
        response_dict['message'] = "Completed with errors."
    else:
        response_dict['code'] = 200
        response_dict['message'] = "OK"

    return response_dict


if __name__ == "__main__":

    parser = argparse.ArgumentParser(
        description='Load in FHIR Bunlde and save Resource entries to an NDJSON file.')
    parser.add_argument(
        dest='bundle_file',
        action='store',
        help='Input the FHIR Bundle.')
    parser.add_argument(
        dest='output_file',
        action='store',
        help="Enter the output NDJSON filename")
    args = parser.parse_args()

    result = fhirbundle2ndjson(args.bundle_file, args.output_file)
    # output the JSON transaction summary
    print(json.dumps(result, indent=4))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Alan Viars

import argparse
import json
import os
import ndjson
import sys
from collections import OrderedDict


def jsondir2ndjson(json_dir, output_filename):
    """Convert all json files under a path into a single ndjson file."""

    fileindex = 0
    error_list = []
    success_index = 0
    response_dict = OrderedDict()
    onlyfiles = []
    try:
        out_fh = open(output_filename, 'w')
        writer = ndjson.writer(out_fh)

        # get the files in the specified directory
        print("Getting a list of all files for importing from", json_dir)
        for root, dirs, files in os.walk(json_dir):
            for file in files:
                if file.endswith(".json") or file.endswith(".js"):
                    onlyfiles.append(os.path.join(root, file))

        for f in onlyfiles:
            j = None
            error_message = ""

            fh = open(f, 'rU')
            fileindex += 1
            j = fh.read()
            fh.close()

            try:
                j = json.loads(j, object_pairs_hook=OrderedDict)
                if not isinstance(j, type(OrderedDict())):
                    error_message = "File " + f + " did not contain a json object, i.e. {}."
                    error_list.append(error_message)
            except:
                error_message = "File " + f + " did not contain valid JSON."
                error_list.append(error_message)

            if not error_message:
                # add the object to the output file.
                try:
                    writer.writerow(j)
                    success_index += 1
                except:
                    error_message = "Error writing " + f + \
                        " to NDJSON. " + str(sys.exc_info())
                    error_list.append(error_message)

        if error_list:
            response_dict['num_files_attempted'] = fileindex
            response_dict['num_files_imported'] = success_index
            response_dict['num_file_errors'] = len(error_list)
            response_dict['errors'] = error_list
            response_dict['code'] = 400
            response_dict['message'] = "Completed with errors."
        else:

            response_dict['num_files_attempted'] = fileindex
            response_dict['num_files_imported'] = success_index
            response_dict['num_file_errors'] = len(error_list)
            response_dict['code'] = 200
            response_dict['message'] = "Completed without errors."

    except:
        response_dict = {}
        response_dict['num_files_attempted'] = fileindex
        response_dict['num_files_imported'] = success_index
        response_dict['code'] = 500
        response_dict['errors'] = [str(sys.exc_info()), ]
        response_dict['message'] = str(sys.exc_info())

    return response_dict


if __name__ == "__main__":

    # Parse args
    parser = argparse.ArgumentParser(
        description='Load in directory which contains JSON docs and convert it into a single NDJSON file.')
    parser.add_argument(
        dest='input_JSON_dir',
        action='store',
        help='Input the JSON dir to load here')
    parser.add_argument(
        dest='output_NDJSON_file',
        action='store',
        help="Enter the output filename")
    args = parser.parse_args()

    result = jsondir2ndjson(args.input_JSON_dir, args.output_NDJSON_file)

    # output the JSON transaction summary
    print(json.dumps(result, indent=4))



#!/usr/bin/env python


from FITE import app
import argparse
import json
from collections import OrderedDict
import os


DEFAULT_SMART_ON_FHIR_SECURITY_DESCRIPTION = os.getenv("DEFAULT_SMART_ON_FHIR_DESCRIPTION_SECURITY_DESCRIPTION",
                                                       """
1. See the [General Security Considerations](/Authorization_Authentication_and_Registration.html) section for requirements 
and recommendations. 1. A server **SHALL** reject any unauthorized requests by returning an `HTTP 401` unauthorized response code.
"""
                                                       )


def get_scopes():
    scopes = {"openid": "default",
              "fhirUser": "who is logged in",
              "profile": "default",
              "system/*.*": "All interactions allowed for all available resources",
              "system/*.read": "All read interactions allowed for all available resources",
              "system/*.write": "All write interactions allowed for all available resources",
              "patient/*.*": "All interactions allowed for a single patient for all available resources",
              "patient/*.read": "All read interactions allowed for single patient for all available resources",
              "patient/*.write": "All write interactions allowed for single patient for all available resources"
              }
    return scopes


def get_component_parameters():
    parameter_str = \
        """
    {"rid":{
               "name":"rid",
               "in":"path",
               "description":"id of the resource (=Resource.id)",
               "required":true,
               "allowEmptyValue":false,
               "style":"simple",
               "schema":{
                  "type":"string"
               }
            },
            "hid":{
               "name":"hid",
               "in":"path",
               "description":"id of the history entry (=Resource.meta.versionId)",
               "required":true,
               "allowEmptyValue":false,
               "style":"simple",
               "schema":{
                  "type":"string"
               }
            },
            "summary":{
               "name":"_summary",
               "in":"query",
               "description":"Requests the server to return a designated subset of the resource",
               "allowEmptyValue":true,
               "style":"form",
               "schema":{
                  "type":"string",
                  "enum":[
                     "true",
                     "text",
                     "data",
                     "count",
                     "false"
                  ]
               }
            },
            "format":{
               "name":"_format",
               "in":"query",
               "description":"Specify alternative response formats by their MIME-types (when a client is unable acccess accept: header)",
               "allowEmptyValue":true,
               "style":"form",
               "schema":{
                  "type":"string",
                  "format":"mime-type"
               }
            },
            "pretty":{
               "name":"_pretty",
               "in":"query",
               "description":"Ask for a pretty printed response for human convenience",
               "allowEmptyValue":true,
               "style":"form",
               "schema":{
                  "type":"boolean"
               }
            },
            "elements":{
               "name":"_elements",
               "in":"query",
               "description":"Requests the server to return a collection of elements from the resource",
               "allowEmptyValue":true,
               "style":"form",
               "explode":false,
               "schema":{
                  "type":"array",
                  "format":"string",
                  "items":{
                     "format":"string"
                  }
               }
            },
            "count":{
               "name":"_count",
               "in":"query",
               "description":"The maximum number of search results on a page. The server is not bound to return the number requested, but cannot return more",
               "schema":{
                  "type":"number"
               }
            }
         }
    """
    return json.loads(parameter_str, object_pairs_hook=OrderedDict)


def get_metadata():
    metadata_str = \
        """{"summary": "Access to the Server's Capability Statement",
     "description": "All FHIR Servers return a CapabilityStatement that describes what services they perform",
     "get": {
         "summary": "Return the server's capability statement",
                "responses": {
                    "default": {
                        "description": "Error, with details",
                        "content": {
                        "application/fhir+json": {
                            "schema": {
                                "$ref": "https://hl7.org/fhir/R4/fhir.schema.json#/definitions/OperationOutcome"
                            }
                        }
                        }
                    },
                    "200": {
                        "description": "the capbility statement",
                        "content": {
                        "application/fhir+json": {
                            "schema": {
                                "$ref": "https://hl7.org/fhir/R4/fhir.schema.json#/definitions/CapabilityStatement"
                            }
                        }
                        }
                    }
                },
                "parameters": [
                    {
                        "$ref":"#/components/parameters/format"
                    },
                    {
                        "$ref":"#/components/parameters/pretty"
                    },
                    {
                        "$ref":"#/components/parameters/summary"
                    },
                    {
                        "$ref":"#/components/parameters/elements"
                    }
                ]
            }
        }"""
    return json.loads(metadata_str, object_pairs_hook=OrderedDict)


def get_interaction_stub(description):
    pass


def build_200_get_response(resource_type_name):
    response_str = \
        """
    {"description": "A RESOURCE_TYPE_NAME with specified resource id.",
     "content":{"application/fhir+json":{"schema":{"$ref":"https://hl7.org/fhir/R4/fhir.schema.json#/definitions/OperationOutcome"}}}}
    """
    response_str = response_str.replace("RESOURCE_TYPE_NAME", resource_type_name)
    response_200 = OrderedDict()
    response_200['200'] = json.loads(response_str, object_pairs_hook=OrderedDict)
    return response_200


def build_eob_specific_parameters():
    # returns a list of objects
    eob_params_str = \
        """
        [
        {
                     "name":"_id",
                     "in":"query",
                     "schema":{
                        "type":"string"
                     },
                     "description":"Logical id of this artifact"
                  },
                  {
                     "name":"patient",
                     "in":"query",
                     "schema":{
                        "type":"string"
                     },
                     "description":"The reference to the patient"
                  },
                  {
                     "name":"_lastUpdated",
                     "in":"query",
                     "schema":{
                        "type":"string",
                        "pattern":"([0-9]([0-9]([0-9][1-9]|[1-9]0)|[1-9]00)|[1-9]000)(-(0[1-9]|1[0-2])(-(0[1-9]|[1-2][0-9]|3[0-1])(T([01][0-9]|2[0-3]):[0-5][0-9]:([0-5][0-9]|60)(\\.[0-9]+)?(Z|(\\+|-)((0[0-9]|1[0-3]):[0-5][0-9]|14:00)))?)?)?"
                     },
                     "description":"When the resource version last changed"
                  },
                  {
                     "name":"type",
                     "in":"query",
                     "schema":{
                        "type":"string"
                     },
                     "description":"The type of the ExplanationOfBenefit"
                  },
                  {
                     "name":"identifier",
                     "in":"query",
                     "schema":{
                        "type":"string"
                     },
                     "description":"The business/claim identifier of the Explanation of Benefit"
                  },
                  {
                     "name":"service-date",
                     "in":"query",
                     "schema":{
                        "type":"string",
                        "pattern":"([0-9]([0-9]([0-9][1-9]|[1-9]0)|[1-9]00)|[1-9]000)(-(0[1-9]|1[0-2])(-(0[1-9]|[1-2][0-9]|3[0-1])(T([01][0-9]|2[0-3]):[0-5][0-9]:([0-5][0-9]|60)(\\.[0-9]+)?(Z|(\\+|-)((0[0-9]|1[0-3]):[0-5][0-9]|14:00)))?)?)?"
                     },
                     "description":"Date of the service for the EOB. The service-date search parameter simplifies search, since a client doesn't need to know that for inpatient and outpatient institutional EOB dates they need to search by billablePeriod.period.start, for a pharmacy EOB by item.servicedDate, and for a professional and non-clinician EOB - by item.servicedPeriod.period.start."
                  }
            ]
            """
    return json.loads(eob_params_str, object_pairs_hook=OrderedDict)


def build_oauth2_security_section(scopes, authorization_server_url, name="smart_on_fhir_oauth2"):
    security = {
        "type": "oauth2",
        "description": "Smart on FHIR implementation of OAuth2",
        "name": name,
        "in": "header",
        "scheme": "bearer",
        "flows": [{
            "authorizationCode": {
                "authorizationUrl": f'{authorization_server_url}/authorize',
                "tokenUrl": f'{authorization_server_url}/token',
                "refreshUrl": f'{authorization_server_url}/refresh',
                "scopes": scopes
            }
        }]
    }
    return security


def build_default_fhir_response():
    default_response = OrderedDict()
    default_json_str = \
        """
    {
        "description": "Error, with details",
        "content": {
            "application/fhir+json": {
                "schema": {
                    "$ref": "https://hl7.org/fhir/R4/fhir.schema.json#/definitions/OperationOutcome"
                }
            }
        }
    }
    """
    to_od_object = json.loads(default_json_str, object_pairs_hook=OrderedDict)
    default_response['default'] = to_od_object
    return default_response


def build_parameters(parameter_key_list):
    parameters = []
    for key in parameter_key_list:
        if key == "rid":
            parameters.append({"$ref": "#/components/parameters/rid"})
        if key == "hid":
            parameters.append({"$ref": "#/components/parameters/hid"})
        if key == "summary":
            parameters.append({"$ref": "#/components/parameters/summary"})
        if key == "format":
            parameters.append({"$ref": "#/components/parameters/format"})
        if key == "pretty":
            parameters.append({"$ref": "#/components/parameters/pretty"})
        if key == "elements":
            parameters.append({"$ref": "#/components/parameters/elements"})
        return parameters


def smart_on_fhir_seciurity(description=DEFAULT_SMART_ON_FHIR_SECURITY_DESCRIPTION):
    return OrderedDict(("type", "apiKey",),
                       ("name", "authorization",),
                       ("in", "header",),
                       ("description", description))


def create_openapi_document_base(info_title, info_description, info_license_name, info_license_url,
                                 version, info_contact_name, info_contact_email, info_contact_url,
                                 externalDocs_url, externalDocs_description, openapi_version="3.0.2"):

    # Create an OpenAPI
    openapi = OrderedDict()

    # info inludes  title, desc, license, version, and contact info
    openapi['openapi'] = openapi_version
    openapi['info'] = OrderedDict()
    openapi['info']['title'] = info_title
    openapi['info']['description'] = info_description
    openapi['info']['license'] = OrderedDict()
    openapi['info']['license']['name'] = info_license_name
    openapi['info']['license']['url'] = info_license_url
    openapi['info']['version'] = version
    openapi['info']['contact'] = OrderedDict()
    openapi['info']['contact']['name'] = info_contact_name
    openapi['info']['contact']['email'] = info_contact_email
    openapi['info']['contact']['url'] = info_contact_url

    # external docs
    openapi['externalDocs'] = OrderedDict()
    openapi['externalDocs']['url'] = externalDocs_url
    openapi['externalDocs']['description'] = externalDocs_description

    # paths stub - other defs will fill out those.
    openapi['paths'] = OrderedDict()
    openapi['paths']['/metadata'] = get_metadata()

    # components stub - other defs will fil out those.
    openapi['components'] = OrderedDict()
    openapi['components']['parameters'] = get_component_parameters()
    # return the ordered dict
    return openapi


def build_openapi_security_for_paths(scopes, security_name="smart_on_fhir_oauth2"):
    security = [{security_name: scopes}, ]
    return security


def add_openapi_paths(openapi, cs, authorization_server_url, fhir_server_base_url,
                      security_name="smart_on_fhir_oauth2"):
    """"Add the paths to the OpenAPI document using the "rest" section forom the FHIR Capability Statement."""
    results = OrderedDict()
    results['openapi'] = openapi
    results['errors'] = []
    add_security_section = False
    for rest in cs['rest']:

        if "security" in rest.keys():
            security = rest['security']
            if "service" in security.keys():
                for service in security['service']:
                    # get coding, text and description
                    # print(service['coding'])
                    for code in service['coding']:
                        if "code" in code.keys():
                            if code['code'] == "SMART-on-FHIR":
                                add_security_section = True

        # find the rest object that represents the server
        if rest["mode"] == "server":
            for resource in rest["resource"]:
                # iterate over every resource and create a paths object for every interaction it has.
                # use existing openapi sample for bootstrap
                for i in resource["interaction"]:
                    if i["code"] == "read":
                        path = "/%s/{rid}" % (resource["type"])
                        results['openapi']['paths'][path] = OrderedDict()

                        # get/read
                        results['openapi']['paths'][path]['get'] = OrderedDict()
                        results['openapi']['paths'][path]['get']['description'] = "Returns a specified %s resource with a given resource id" % (
                            resource["type"])
                        results['openapi']['paths'][path]['get']['responses'] = OrderedDict()
                        results['openapi']['paths'][path]['get']['responses'].update(build_default_fhir_response())
                        results['openapi']['paths'][path]['get']['responses'].update(build_200_get_response(resource["type"]))
                        results['openapi']['paths'][path]['get']['parameters'] = build_parameters(
                            ['rid', 'summary', 'format', 'pretty', 'elements'])
                        scopes = [f'patient/{resource["type"]}.read', ]
                        if add_security_section:
                            results['openapi']['paths'][path]['get']['security'] = build_openapi_security_for_paths(scopes=scopes,
                                                                                                                    security_name="smart_on_fhir_oauth2")
                    elif i["code"] == "vread":
                        # get/vread
                        path = "/%s/{rid}/_history/{hid}" % (resource["type"])
                        results['openapi']['paths'][path] = OrderedDict()
                        results['openapi']['paths'][path]['get'] = OrderedDict()
                        results['openapi']['paths'][path]['get']['description'] = "Returns a specified %s resource with a given resource id and a specific historical version" % (
                            resource["type"])
                        results['openapi']['paths'][path]['get']['responses'] = OrderedDict()
                        results['openapi']['paths'][path]['get']['responses'].update(build_default_fhir_response())
                        results['openapi']['paths'][path]['get']['responses'].update(build_200_get_response(resource["type"]))
                        results['openapi']['paths'][path]['get']['parameters'] = build_parameters(
                            ['rid', 'hid', 'summary', 'format', 'pretty', 'elements'])
                        # Add security
                        scopes = [f'patient/{resource["type"]}.read', ]
                        if add_security_section:
                            results['openapi']['paths'][path]['get']['security'] = build_openapi_security_for_paths(scopes=scopes,
                                                                                                                    security_name="smart_on_fhir_oauth2")

                    elif i["code"] == "history-instance":
                        # get/history-instance
                        path = "/%s/{rid}/_history" % (resource["type"])
                        results['openapi']['paths'][path] = OrderedDict()
                        results['openapi']['paths'][path]['get'] = OrderedDict()
                        results['openapi']['paths'][path]['get']['description'] = "Returns a bundle of all versions for the %s resource with given resource id" % (
                            resource["type"])
                        results['openapi']['paths'][path]['get']['responses'] = OrderedDict()
                        results['openapi']['paths'][path]['get']['responses'].update(build_default_fhir_response())
                        results['openapi']['paths'][path]['get']['responses'].update(build_200_get_response(resource["type"]))
                        results['openapi']['paths'][path]['get']['parameters'] = build_parameters(
                            ['rid', 'summary', 'format', 'pretty', 'elements'])
                        # Add security
                        scopes = [f'patient/{resource["type"]}.read', ]
                        if add_security_section:
                            results['openapi']['paths'][path]['get']['security'] = build_openapi_security_for_paths(scopes=scopes,
                                                                                                                    security_name="smart_on_fhir_oauth2")
                    elif i["code"] == "history-type":
                        # get/history-type
                        path = "/%s/_history" % (resource["type"])
                        results['openapi']['paths'][path] = OrderedDict()
                        results['openapi']['paths'][path]['get'] = OrderedDict()
                        results['openapi']['paths'][path]['get']['description'] = "Returns a bundle of all versions for the %s resource that match provided search parameters" % (
                            resource["type"])
                        results['openapi']['paths'][path]['get']['responses'] = OrderedDict()
                        results['openapi']['paths'][path]['get']['responses'].update(build_default_fhir_response())
                        results['openapi']['paths'][path]['get']['responses'].update(build_200_get_response(resource["type"]))
                        results['openapi']['paths'][path]['get']['parameters'] = build_parameters(
                            ['summary', 'format', 'pretty', 'elements'])
                        # Add the search Parameters
                        if "searchParam" in i.keys():
                            for param in i["searchParam"]:
                                results['openapi']['paths'][path]['get']['parameters'].append({
                                    "$ref": f'#components/parameters/{param["name"]}'
                                })
                        # Add security
                        scopes = [f'patient/{resource["type"]}.read', ]
                        if add_security_section:
                            results['openapi']['paths'][path]['get']['security'] = build_openapi_security_for_paths(scopes=scopes,
                                                                                                                    security_name="smart_on_fhir_oauth2")
                    elif i["code"] == "search-type":
                        # get/search-type
                        path = "/%s" % (resource["type"])
                        results['openapi']['paths'][path] = OrderedDict()
                        results['openapi']['paths'][path]['get'] = OrderedDict()
                        results['openapi']['paths'][path]['get']['description'] = "Returns a bundle of the %s resource that match provided search parameters" % (
                            resource["type"])
                        results['openapi']['paths'][path]['get']['responses'] = OrderedDict()
                        results['openapi']['paths'][path]['get']['responses'].update(build_default_fhir_response())
                        results['openapi']['paths'][path]['get']['responses'].update(build_200_get_response(resource["type"]))
                        results['openapi']['paths'][path]['get']['parameters'] = build_parameters(
                            ['summary', 'format', 'pretty', 'elements'])
                        if "searchParam" in i.keys():
                            for param in i["searchParam"]:
                                results['openapi']['paths'][path]['get']['parameters'].append({
                                    "$ref": f'#components/parameters/{param["name"]}'
                                })
                        # Add security
                        scopes = [f'patient/{resource["type"]}.read', ]
                        if add_security_section:
                            results['openapi']['paths'][path]['get']['security'] = build_openapi_security_for_paths(scopes=scopes,
                                                                                                                    security_name="smart_on_fhir_oauth2")
                    # continue to add each of the following interactions
                    # elif i["code"] == "update":
                    # elif i["code"] == "patch":
                    # elif i["code"] == "delete":
                    # elif i["code"] == "create":
                    else:
                        # skip system interactions (not related to any resource)
                        pass
        else:
            # ignoring the client object, if its even there (unlikely)
            pass
    return results


def add_openapi_components(openapi, cs, authorization_server_url,
                           scopes=get_scopes(), security_name="smart_on_fhir_oauth2"):
    """Add the comentents to the OpenAPI document using the "rest" section forom the FHIR Capability Statement."""
    results = OrderedDict()
    results['openapi'] = openapi
    results['errors'] = []
    # restpart = cs['rest']
    # print("Create OpenAPI components.")
    results['openapi']['components']['securitySchemes'] = build_oauth2_security_section(scopes,
                                                                                        authorization_server_url=authorization_server_url,
                                                                                        name=security_name)
    return results


def open_and_read_cap_statement(input_capability_statement_file):
    cs = OrderedDict()
    # Open and read in the CS and/or config file containing a CS
    fh = open(input_capability_statement_file, 'rU')
    j = fh.read()
    j = json.loads(j, object_pairs_hook=OrderedDict)

    # Make sure it is what we are looking for
    if not isinstance(j, type(OrderedDict())):
        error_message = "File " + \
            str(input_capability_statement_file) + " did not contain a JSON object, i.e. {}."
        print(error_message)
        raise Exception(error_message)
    # Get the cap statement from config file or as raw resource file.
    if 'CapabilityStatement' in j.keys():
        cs = j['CapabilityStatement']
    if 'resourceType' in j:
        if j['resourceType'] == 'CapabilityStatement':
            cs = j
    """Open the file and fetch the Capability statement"""
    return cs


def capstatement2swagger(cs, authorization_server_url="https://api.example.com",
                         fhir_server_base_url="https://fhir.example.com/baseR4",
                         security_name="smart_on_fhir_oauth2"):
    """Input an OrderedDict containing a FHIR Capabaility Statement"""
    # Error list
    error_list = []

    result = OrderedDict()
    result['openapi'] = OrderedDict()
    # Get the contact info from the list
    info_contact_name = ""
    info_contact_email = ""
    info_contact_url = ""
    info_title = ""
    info_description = ""
    if "description" in cs.keys():
        info_description = cs["description"]
    if "title" in cs.keys():
        info_title = cs["title"]
    elif "name" in cs.keys():
        info_title = cs["name"]
    else:
        info_title = "DRAFT"

    for contact in cs['contact']:
        if 'telecom' in contact.keys():
            for telecom in contact['telecom']:
                if 'system' in telecom.keys():
                    if telecom['system'] == 'email':
                        info_contact_email = telecom['value']
                    if telecom['system'] == 'url':
                        info_contact_url = telecom['value']

    # Get the external document URL form the first list element in the CS
    externalDocs_url = ""
    if 'implementationGuide' in cs.keys():
        for ig in cs['implementationGuide']:
            externalDocs_url = ig
            break

    # create the base openAPI object
    openapi = create_openapi_document_base(info_title=info_title, info_description=info_description,
                                           info_license_name=app.config("CS_INFO_LICENSE_NAME",
                                                                        "Creative Commons Zero v1.0 Universal"),
                                           info_license_url=app.config("CS_INFO_LICENSE_URL",
                                                                       "http://spdx.org/licenses/CC0-1.0.html"),
                                           version=cs['version'],
                                           info_contact_name=info_contact_name,
                                           info_contact_email=info_contact_name,
                                           info_contact_url=info_contact_url,
                                           externalDocs_url=externalDocs_url,
                                           externalDocs_description=app.config("CS_EXTERNAL_DOCS_DESCRIPTION", "External Docs Description"),)

    # Add the "components" part including the security
    openapi_results = add_openapi_components(openapi, cs, authorization_server_url,
                                             get_scopes(), security_name=security_name)

    # Add the "paths" part
    openapi_results = add_openapi_paths(openapi_results['openapi'], cs, authorization_server_url,
                                        fhir_server_base_url, security_name=security_name)

    return openapi_results


if __name__ == "__main__":
    # Parse args
    parser = argparse.ArgumentParser(description='Load in Capability Statement')

    parser.add_argument(
        dest='input_capability_statement_file',
        action='store',
        help='Input the Capability Statement file to load here.')

    parser.add_argument(
        dest='output_openapi_file',
        action='store',
        help="Enter the output filename.  This is the openapi document suitable for Swagger.")

    parser.add_argument('--auth_server',
                        dest='authorization_server_url',
                        action='store',
                        help='Authorization server base url.',
                        default='https://api.example.com')

    parser.add_argument('--fhir_server_base',
                        dest='fhir_server_base_url',
                        action='store',
                        help='FHIR Base URL.',
                        default='https://fhir.example.com/baseR4')

    args = parser.parse_args()

    result = capstatement2swagger(open_and_read_cap_statement(args.input_capability_statement_file),
                                  args.authorization_server_url, args.fhir_server_base_url)

    fh = open(args.output_openapi_file, 'w')
    fh.write(json.dumps(result['openapi'], indent=4))
    fh.close()
    # output errors (if any).
    if result['errors']:
        print("%s errors occured while creating the OpenAPI file from the FHIR Capabaility statement",
              len(result['errors']))
        for e in result['errors']:
            print(e)

    #print(json.dumps(result, indent=4))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4


import json
import sys
from collections import OrderedDict
from pymongo import MongoClient


def json2mongo(jsonfile, database_name, collection_name,
               delete_collection_before_import, host, port):
    """Return a response_dict with a summary of json2mongo transaction."""
    # print "Start the import of", jsonfile, "into the collection",
    # collection_name, "within the database", database_name, "."

    response_dict = OrderedDict()
    fileindex = 0
    mongoindex = 0
    error_list = []

    mc = MongoClient(host=host, port=port)
    db = mc[database_name]
    collection = db[collection_name]

    if delete_collection_before_import:
        db.drop_collection(collection)
    fh = open(jsonfile, 'rU')

    j = fh.read()

    try:
        j = json.loads(j, object_pairs_hook=OrderedDict)

        if not isinstance(j, type(OrderedDict())):
            error_message = "File " + \
                str(jsonfile) + " did not contain a JSON object, i.e. {}."
            error_list.append(error_message)

    except:
        error_message = "File " + \
            str(jsonfile) + " did not contain valid JSON."
        error_list.append(error_message)

    if not error_list:

        myobjectid = collection.insert(j)
        mongoindex += 1

        if error_list:
            response_dict['num_files_imported'] = mongoindex
            response_dict['num_file_errors'] = len(error_list)
            response_dict['errors'] = error_list
            response_dict['code'] = 400
            response_dict['message'] = "Completed with errors."

        else:
            response_dict['num_rows_imported'] = mongoindex
            response_dict['num_file_errors'] = len(error_list)
            response_dict['code'] = 200
            response_dict['message'] = "Completed without errors."

    else:
        response_dict['num_rows_imported'] = mongoindex
        response_dict['num_file_errors'] = len(error_list)
        response_dict['code'] = 500
        syserror = sys.exc_info()
        errors = error_list
        if syserror[0]:
            errors.append(str(sys.exc_info()))

        response_dict['errors'] = errors

    return response_dict



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Alan Viars

import argparse
import json
from collections import OrderedDict
from pymongo import MongoClient
import ndjson


def ndjson2mongo(ndjsonfile, database_name,
                 collection_name,
                 delete_collection_before_import,
                 host,
                 port):
    """Return a response_dict with a summary of ndjson2mongo transaction."""
    print("Import file", ndjsonfile,
          "into the MongoDB collection", collection_name, ".")
    # collection_name, "within the database", database_name, "."

    response_dict = OrderedDict()
    fileindex = 0
    mongoindex = 0
    error_list = []

    mc = MongoClient(host=host, port=port)
    db = mc[database_name]
    collection = db[collection_name]

    if delete_collection_before_import:
        db.drop_collection(collection)

    with open(ndjsonfile) as f:
        data = ndjson.load(f, object_pairs_hook=OrderedDict)
        for item in data:
            # print("item",item)
            try:
                if not isinstance(item, type(OrderedDict())):
                    error_message = "File " + \
                        str(ndjsonfile) + \
                        " did not contain a JSON object, i.e. {}."
                    error_list.append(error_message)
                # insert the item/document
                myobjectid = collection.insert(item)
                mongoindex += 1

            except:
                # print(sys.exc_info())
                error_message = "File " + \
                    str(item) + " did not contain valid JSON."
                error_list.append(error_message)

        if error_list:
            response_dict['file'] = ndjsonfile
            response_dict['database'] = database_name
            response_dict['collection'] = collection_name
            response_dict['num_files_imported'] = mongoindex
            response_dict['num_file_errors'] = len(error_list)
            response_dict['errors'] = error_list
            response_dict['code'] = 400
            response_dict['message'] = "Completed with errors."

        else:
            response_dict['file'] = ndjsonfile
            response_dict['database'] = database_name
            response_dict['collection'] = collection_name
            response_dict['num_rows_imported'] = mongoindex
            response_dict['num_file_errors'] = len(error_list)
            response_dict['code'] = 200
            response_dict['message'] = "Completed without errors."

    return response_dict

if __name__ == "__main__":

    # Parse args
    parser = argparse.ArgumentParser(
        description='Load in the NDJSON doc into MongoDB')
    parser.add_argument(
        dest='input_ndjson_file',
        action='store',
        help='Input the NDJSON file to load here')
    parser.add_argument(
        dest='db_name',
        action='store',
        help="Enter the Database name you want to import the JSON to")
    parser.add_argument(
        dest='collection_name',
        action='store',
        help="Enter the Collection name within the Database specified that you want the JSON to be imported to")
    parser.add_argument('-d', '--delete', dest='delete', action='store_true',
                        help='Delete previous collection upon import')
    parser.add_argument(
        '--host',
        dest='host',
        action='store',
        default='127.0.0.1',
        help='Specify host. Default is 127.0.0.1 ')
    parser.add_argument(
        '-p',
        '--port',
        dest='port',
        action='store',
        default=27017,
        help='Specify port. Default is 27017')
    args = parser.parse_args()
    ndjson_file = args.input_ndjson_file
    database = args.db_name
    collection = args.collection_name
    delete_collection_before_import = args.delete
    host = args.host
    port = args.port

    result = ndjson2mongo(
        ndjson_file,
        database,
        collection,
        delete_collection_before_import,
        host,
        port)

    # output the JSON transaction summary
    print(json.dumps(result, indent=4))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4


__author__ = "Alan Viars"
from collections import OrderedDict
import ndjson
import requests
import sys
import uuid
import json
from datetime import datetime, timedelta


def clean_list(mylist):
    if isinstance(mylist, type([])):
        new_list = []
        for i in mylist:
            if isinstance(i, str):
                new_list.append(i.strip())
            elif isinstance(i, type(OrderedDict())) or isinstance(i, dict):
                new_list.append(clean_dict(i))
            elif isinstance(i, type([])):
                new_list.append(clean_list(i))
            else:
                new_list.append(i)
    return new_list


def clean_dict(item):
    """Recursively find strings and strip them """
    cleaned_item = OrderedDict()
    if isinstance(item, type(OrderedDict())) or isinstance(item, dict):
        for key, value in item.items():
            if isinstance(value, str):
                cleaned_item[key] = value.strip()
                if not cleaned_item[key]:
                    # remove empty strings
                    del cleaned_item[key]
            elif isinstance(value, type(None)):
                # do not add nulls
                pass
            elif isinstance(value, type([])):
                cleaned_item[key] = clean_list(value)
            elif isinstance(value, type(OrderedDict())) or isinstance(value, dict):
                cleaned_item[key] = clean_dict(value)
            else:  # bool
                cleaned_item[key] = value
            cleaned_item = item
    return cleaned_item


def ndjson2fhir(ndjsonfile, fhir_base_url, update=True,
                output_http_response=False,
                oauth2_token=None,
                authorization_uri=None, client_id=None, client_secret=None,
                resource=None, refresh_access_token_in_minutes=57):
    """Return a response_dict with a summary of ndjson2fhir transaction."""
    print("Starting the import of", ndjsonfile, "into", fhir_base_url, "...")
    response_dict = OrderedDict()
    index = 1
    error_list = []
    ids = []
    failed_lines = []
    with open(ndjsonfile) as f:
        data = ndjson.load(f, object_pairs_hook=OrderedDict)
        first_run = True
        start_time = datetime.now()
        next_token_refesh_time = start_time + timedelta(minutes=refresh_access_token_in_minutes)
        for item in data:
            # Check the age of the token.

            # if not access_token and a authorization_uri is supplied, attempt to get an access_token
            if next_token_refesh_time < datetime.now() or first_run:
                if authorization_uri and client_id and client_secret:
                    # Get the access_token
                    print("Getting access token")
                    payload = {'client_id': client_id,
                               'client_secret': client_secret,
                               'grant_type': 'client_credentials'}
                    if resource:
                        payload['resource'] = resource
                    # Do POST to get authz endpoint
                    r = requests.post(authorization_uri, data=payload)
                    jsr = json.loads(r.content)
                    if output_http_response:
                        print("Access token response", r.content)
                    oauth2_token = jsr['access_token']
                    next_token_refesh_time = datetime.now() + timedelta(minutes=refresh_access_token_in_minutes)
                    first_run = False

            try:
                if not isinstance(item, type(OrderedDict())):
                    error_message = "File " + \
                        str(item) + " did not contain a JSON object, i.e. {}."
                    error_list.append(error_message)

                # Clean the strings in the JSON Dict of whitespace padding and blank strings.
                item = clean_dict(item)

                # insert the item/document using an HTTP POST to a FHIR server
                if update:
                    # A PUT
                    if 'id' not in item.keys():
                        item['id'] = str(uuid.uuid4())
                    resource_url = "%s%s/%s" % (fhir_base_url, item["resourceType"], item['id'])
                else:
                    # A POST
                    if 'id' in item.keys():
                        del item['id']
                    resource_url = "%s%s" % (fhir_base_url, item["resourceType"])

                headers = {'Content-type': 'application/json'}

                # Set the Bearer token
                if oauth2_token:
                    headers["Authorization"] = "Bearer %s" % (oauth2_token)

                if update:
                    # PUT UPDATE
                    r = requests.put(resource_url, json=item, headers=headers)

                    if r.status_code not in (200, 201,):
                        print
                        error_message = "Failed to PUT/UPDATE item number %s in the FHIR server." % (index)
                        error_list.append(error_message)
                        failed_lines.append(index)
                    else:
                        ids.append({index: r.json()['id']})
                else:
                    # POST CREATE
                    r = requests.post(resource_url, json=item, headers=headers)
                    if r.status_code not in (201,):
                        error_message = "Failed to POST/CREATE item number %s in the FHIR server." % (index)
                        error_list.append(error_message)
                        failed_lines.append(index)
                    else:
                        ids.append({index: r.json()['id']})
                # Output the response if requested.
                if output_http_response:
                    print("index=", index, "url=", resource_url, "status_code=",
                          r.status_code, "content=", r.content)
            except Exception:
                print(sys.exc_info())
                error_message = "File " + str(item) + str(sys.exc_info())
                error_list.append(error_message)
                failed_lines.append(index)
            index += 1
        if error_list:
            response_dict['file'] = ndjsonfile
            response_dict['fhir_base_url'] = fhir_base_url
            response_dict['num_fhir_resources_created'] = index - 1
            response_dict['ids'] = ids
            response_dict['num_upload_errors'] = len(error_list)
            response_dict['errors'] = error_list
            response_dict['falied_lines'] = failed_lines
            response_dict['code'] = 400
            response_dict['message'] = "Completed with errors."

        else:
            response_dict['file'] = ndjsonfile
            response_dict['fhir_base_url'] = fhir_base_url
            response_dict['num_fhir_resources_created'] = index - 1
            response_dict['ids'] = ids
            response_dict['num_file_errors'] = len(error_list)
            response_dict['falied_lines'] = failed_lines
            response_dict['code'] = 200
            response_dict['message'] = "Completed. No errors."

    return response_dict



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Alan Viars

import csv
import sys
import traceback
from collections import OrderedDict
from pymongo import MongoClient


def csv2mongo(csvfile, database_name, collection_name, delete_collection_before_import, host, port):
    """Return a response_dict with summary of csv2mongo transaction."""

    response_dict = OrderedDict()

    try:

        mc = MongoClient(host=host, port=port)
        db = mc[database_name]
        collection = db[collection_name]

        if delete_collection_before_import:
            db.drop_collection(collection)

        # open the csv file.
        csvhandle = csv.reader(open(csvfile, 'r'), delimiter=',')

        rowindex = 0
        mongoindex = 0
        error_list = []

        for row in csvhandle:

            if rowindex == 0:
                column_headers = row
                cleaned_headers = []
                for c in column_headers:
                    c = c.replace(".", "")
                    c = c.replace("(", "")
                    c = c.replace(")", "")
                    c = c.replace("$", "-")
                    c = c.replace(" ", "_")
                    cleaned_headers.append(c)
            else:

                record = OrderedDict(zip(cleaned_headers, row))
                # for k,v in flat_record.items():
                #            if v:
                #                kwargs[k]=v

                try:
                    myobjectid = collection.insert(record)
                    mongoindex += 1

                except:
                    error_message = "Error on row " + \
                        str(rowindex) + ". " + str(sys.exc_info())
                    error_list.append(error_message)

            rowindex += 1

        if error_list:
            response_dict['num_rows_imported'] = rowindex
            response_dict['num_rows_errors'] = len(error_list)
            response_dict['errors'] = error_list
            response_dict['code'] = 400
            response_dict['message'] = "Completed with errors"
        else:

            response_dict['num_rows_imported'] = mongoindex
            response_dict['num_csv_rows'] = rowindex
            response_dict['code'] = 200
            response_dict['message'] = "Completed."

    except:
        response_dict['code'] = 500
        response_dict['errors'] = [traceback.print_exc()]

    return response_dict



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Alan Viars

import argparse
import json
import sys
from collections import OrderedDict
from pymongo import MongoClient

def jsonFHIRBundle2mongo(jsonfile, database_name,
                 delete_database_before_import,
                 host,
                 port,
                 collection_name_prefix=''):
    
    """Inserts Bundle as separate resources."""
    print("Opening JSON  file:", jsonfile)
    print("NOTE: Depending on your computer/configuration, large files may need to be split.")
    print("""If you get a "Process killed" in AWS, try increasing the machine's RAM/CPU.""")
    print("For very large files, export entry list into their own files or NDJSON.")

    response_dict = OrderedDict()
    mongoindex = 0
    error_list = []

    mc = MongoClient(host=host, port=port)
    db = mc[database_name]

    if delete_database_before_import:
        print("Drop the database...", database_name)
        mc.drop_database(database_name)
    
    print("""Opening JSON file and expecting a FHIR Bundle or at least an object with an "entry" list with more objects..""")
    print("Loading the JSON file into memory...")
    with open(jsonfile) as f:
        
        item = json.loads(f.read(), object_pairs_hook=OrderedDict)
        print("JSON loaded successfully...")
        if not isinstance(item, type(OrderedDict())):
            error = "Not a JSON object."
            error_list.append(error)
            response_dict['num_errors'] = len(error_list)
            print(error)
            response_dict['errors'] = error_list
            response_dict['code'] = 400            
            response_dict['message'] = "Completed with errors."
            return response_dict

        print("Object loaded successfully...")
        
        response_dict['inputfile'] = jsonfile
        response_dict['database'] = database_name
        response_dict['num_entries_imported'] = mongoindex
        response_dict['fhir_resources_included'] = []
        response_dict['num_errors'] = 0
        
        if "entry" not in item.keys():
            error = "Failed to find entry [] list."
            print(error)
            error_list.append(error)
            response_dict['num_errors'] = len(error_list)
            response_dict['errors'] = error_list
            response_dict['code'] = 400            
            response_dict['message'] = "Completed with errors."
            return response_dict
        else:
            print("Processing %s items." % (len(item['entry'])))
        i=0
        for e in item['entry']:
            try:
                #print(mongoindex,e['resource']["resourceType"])
                collection_name = "%s%s" % (collection_name_prefix,e['resource']["resourceType"])
                collection = db[collection_name]    
                myobjectid = collection.insert(e)
                mongoindex += 1
                response_dict['num_entries_imported'] = mongoindex
                response_dict['num_errors'] = len(error_list)
            except Exception:
                print(sys.exc_info())
                error_message = "Entry %s failed. - %s" % (e, sys.exc_info())
                error_list.append(error_message)
            i += 1

        response_dict['inputfile'] = jsonfile
        response_dict['database'] = database_name
        response_dict['errors'] = error_list
        response_dict['num_entries_imported'] = mongoindex
        response_dict['num_errors'] = len(error_list)
        
        if response_dict['num_errors']:
            response_dict['code'] = 400
            response_dict['message'] = "Completed with errors."
        else:
            response_dict['code'] = 200
            response_dict['message'] = "OK"
    return response_dict

if __name__ == "__main__":

    # Parse args
    parser = argparse.ArgumentParser(
        description='Input JSON FHIR Bundle file.')
    parser.add_argument(
        dest='input_json_file',
        action='store',
        help='Input the JSON file to load here.')
    parser.add_argument(
        dest='db_name',
        action='store',
        help="Enter the Database name you want to import the JSON to")
    parser.add_argument('-d', '--delete', dest='delete', action='store_true',
                        help='Delete previous collection upon import')
    parser.add_argument(
        '--host',
        dest='host',
        action='store',
        default='127.0.0.1',
        help='Specify host. Default is 127.0.0.1 ')
    parser.add_argument(
        '-c', '--collection_name_prefix',
        dest='cnp',
        action='store',
        default='',
        help='Specify a prefix for FHIR resource collection names.Default is blank')
    parser.add_argument(
        '-p',
        '--port',
        dest='port',
        action='store',
        default=27017,
        help='Specify port. Default is 27017')
    args = parser.parse_args()
    json_file = args.input_json_file
    database = args.db_name
    delete_database_before_import = args.delete
    collection_name_prefix = args.cnp
    host = args.host
    port = args.port

    result = jsonFHIRBundle2mongo(
        json_file,
        database,
        delete_database_before_import,
        host,
        port,
        collection_name_prefix)

    # output the JSON transaction summary
    print(json.dumps(result, indent=4))